@@ -1,4 +1,4 @@
-
+{-DHUN| module to download images form the wiki DHUN-}
 module GetImages where
 import ImperativeState
 import UrlAnalyse
@@ -63,10 +63,13 @@ getImageUrl fi u ss
         fil = BStr.unpack (UTF8Str.fromString fi)
         theHref = BStr.unpack (UTF8Str.fromString "href=\"")
         q = BStr.unpack (UTF8Str.fromString "\"")
+
+
+{-DHUN| downloads a single image form the wiki.  It takes the temporary image download directory as first parameter. It takes the WikiUrl of the wiki websitze currently being processed as second parameter. It takes a tuple as third input parameter. The first element of the tuple is the image number so just an integer that can be used to identify the image uniquely) . The second element of the tupele is image include string of the image from the wiki source, that is the text in between the square brackets as second input parameter. It returns a tuple. The first element of the tuple is a list of urls under which the image may be found on the wiki. The second element of the tuple is the image number as described above. The third element of the tuple is the Url where the description page of the image on the wiki is located DHUN-}
  
 getImagePage ::
              String ->
-               WikiUrl -> (Integer, String) -> IO (Maybe ([String], Integer))
+               WikiUrl -> (Integer, String) -> IO (Maybe ([String], Integer, URL))
 getImagePage dir u (i, ss)
   = do l <- (mapM (geturl . unify . exportURL . modpath2 ss)
                (parses u))
@@ -76,35 +79,40 @@ getImagePage dir u (i, ss)
        let yy = (map go gg) :: [[(URL, String)]]
        let zz = (listToMaybe (concat yy)) :: Maybe (URL, String)
        case zz of
-           Just (_, x) -> do img <- (geturl2 x) :: (IO BStr.ByteString)
-                             BStr.writeFile (dir </> (show i)) img
-                             return
-                               (Just (map (unify . exportURL . (modpath2 ss)) (parses u), i))
+           Just (du, x) -> do img <- (geturl2 x) :: (IO BStr.ByteString)
+                              BStr.writeFile (dir </> (show i)) img
+                              return
+                                (Just (map (unify . exportURL . (modpath2 ss)) (parses u), i, modpath2 ss du))
            _ -> return Nothing
   where  
         go :: (URL, Maybe String) -> [(URL, String)]
         go (uu, Just x) = [(uu, x)]
         go _ = []
  
+
+{-DHUN| downloads a single image form the wiki.  It takes the temporary image download directory as first parameter. It takes a tuple as second input parameter. The first element of the tuple is the image number so just an integer that can be used to identify the image uniquely) . The second element of the tupele is image include string of the image from the wiki source, that is the text in between the square brackets as second input parameter. It takes the WikiUrl of the wiki websitze currently being processed as thrird parameter. See function getImages in this module for documentation on the returned data type DHUN-}
+
 doImage ::
         String ->
           WikiUrl ->
-            (Integer, String) -> IO [Maybe (String, Integer, [String])]
+            (Integer, String) -> IO (Maybe ImageInfo)
 doImage dir theWikiUrl img
   = do p <- getImagePage dir theWikiUrl (fst img, theName)
        case p of
-           Just (u, pp) -> return [Just (theName, pp, u)]
-           _ -> return [Nothing]
+           Just (u, pp, du) -> return (Just ImageInfo{wikiFilename=theName,imageNumber=pp,contributorUrls=u,descriptionUrl=du})
+           _ -> return Nothing
   where theName
           = case dropWhile (/= ':') (takeWhile (/= '|') (snd img)) of
                 (_ : xs) -> xs
                 _ -> []
  
+{-DHUN| main function to download images. It takes the temporary image download directory as first parameter. It takes image include strings of the images from the wiki source, that is the text in between the square brackets as second input parameter. It takes the WikiUrl of the wiki websitze currently being processed as thrird parameter. This function runs as a background process. So it returns a list of empty MVars immediately when being called. They are later on filled with information on the downloaded images including their location in the temporary image download directory. The returend MVars contain ImageInfo values. See description in the module ImperativeState for a detailed description. DHUN-}
+ 
 getImages ::
           String ->
             [String] ->
               WikiUrl ->
-                ImperativeMonad [MVar [(Maybe (String, Integer, [String]))]]
+                ImperativeMonad [MVar (Maybe ImageInfo)]
 getImages dir images theWikiUrl
   = liftIO $
       do ddir <- base dir
