@@ -1,4 +1,4 @@
-
+{-DHUN| module defining the datatypes needed for the outer imperative flow control of the program. All configurtion information needed for single run of the program is stored here. DHUN-}
 module ImperativeState where
 import Data.Map
 import Control.Monad.State
@@ -6,13 +6,14 @@ import Control.Monad.Error
 import UrlAnalyse
 import Control.Concurrent.MVar
 import Data.List
- 
+import Network.URL
 data MyError = DownloadError String String
              | OtherError String
              | WikiUrlParseError String
              | NotImplementedError
              | NotExcatlyOneError String
              | NotIntegerError String
+             | NotIntegerPairError String
              | NotAtMostOneError String
              | ToManyOptionsError
              | PaperError
@@ -29,6 +30,8 @@ instance Show MyError where
           = "Error: The supplied url " ++ theUrl ++ " could not be parsed"
         show NotImplementedError
           = "Error: The requested feature is not implemented yet"
+        show (NotIntegerPairError msg)
+          = "Error: The option --" ++ msg ++ "could not be parsed to a pair of integers (like -f 23:42)"
         show PaperError
           = "Error: The option paper may only be one of A4,A5,B5,letter,legal,executive"
         show ToManyOptionsError
@@ -62,6 +65,9 @@ imperativeStateZero
 data ImperativeState = ImperativeState{audict ::
                                        [MVar (Map String Contributor)],
                                        fullUrl :: FullWikiUrl, tmpPath :: String}
+
+data ImageInfo = ImageInfo{wikiFilename :: String,imageNumber::Integer,contributorUrls::[String],descriptionUrl::URL}
+
  
 type ImperativeMonad = ErrorT MyError (StateT ImperativeState IO)
  
@@ -75,8 +81,11 @@ data SourceMode = Included
                 | Excluded
                 deriving (Show, Read)
  
-data FullConfig = FullConfig{resolution :: Integer,
-                             outputFilename :: String, inputUrl :: String, runMode :: RunMode,
-                             paper :: String, vector :: Bool, copy :: Maybe String,
-                             mainPath :: String, server :: Maybe Int}
+data OutputType = PlainPDF | ZipArchive
+                deriving (Show, Read,Eq)
+data FullConfig = FullConfig{headers :: Maybe String,
+                             resolution :: Integer, outputFilename :: String,
+                             inputUrl :: String, runMode :: RunMode, paper :: String,
+                             vector :: Bool, copy :: Maybe String, mainPath :: String,
+                             server :: Maybe Int, outputType::OutputType, selfTest ::Maybe (Integer,Integer)}
                 deriving (Show, Read)
