@@ -1,4 +1,4 @@
-
+{-DHUN| module to download the wiki source text form the wiki website DHUN-}
 module Load where
 import ImperativeState
 import Tools
@@ -19,7 +19,6 @@ import SimpleContributors
 import WikiHelper
 import MediaWikiParseTree
 import MediaWikiParser
-import Compiler
 import Network.URL
 import Control.Concurrent
 import Parallel
@@ -116,8 +115,10 @@ includeAction = qIncludeAction
  
 qIncludeAction :: WikiUrl -> String -> ImperativeMonad String
 qIncludeAction wurl text
-  = do pp <- liftIO (getpage d (wurl))
-       case pp of
+  = if isInfixOf "Vorlage" text then return ("{{"++text++"}}")
+     else  
+      do pp <- liftIO (getpage d (wurl))
+         case pp of
            Just p -> do _ <- addContributors d Nothing
                         noinclude wurl ("\n\ndhunparserurl " ++ d ++ "\n\n" ++ p)
            _ -> return ""
@@ -324,7 +325,11 @@ loadPlain st uu
 loadHTML :: ImperativeState -> ImperativeMonad String
 loadHTML st
   = let fu = fullUrl st in
-      do _ <- loadPlain st (Just (url fu))
+      do midst<-get
+         (res,newst)<-liftIO (runStateT (runErrorT (loadPlain st (Just (url fu)))) midst)
+         case res of 
+           Right _->put newst
+           _ -> return ()    
          x <- liftIO (geturl2 (exportURL (url fu)))
          return . decode . unpack $ x
  
@@ -356,6 +361,8 @@ loadMediaWiki st
                                               (DownloadError (lemma fu) (exportURL (url fu)))
              _ -> throwError (DownloadError (lemma fu) (exportURL (url fu)))
  
+{-DHUN| main function to download images form the wiki. It takes the RunMode as only parameter. In case of HTML the html from the website is loaded. In all other cases the wiki source text is downloaded. In case of ExpandedTemplates the templates are also expanded by mediawiki running on the wiki website DHUN-}
+ 
 load :: RunMode -> ImperativeMonad String
 load theRunMode
   = do st <- get
