@@ -511,15 +511,18 @@ htmlchars
 getHtmlChar :: String -> String
 getHtmlChar x = Map.findWithDefault [] x (Map.fromList htmlchars)
  
+{-DHUN| a function to remove the taling print version string from urls (usually on wikibooks). This function is needed so the name of the book without the tailing print version string will be printed on the titlepage of the book DHUN-}
+ 
 removePrintVersion :: [Char] -> [Char]
 removePrintVersion lem
   = fun
       ["/Print version", "/Complete Wikibook", "/All Chapters",
-       "/Print Version", "/print version", "/Printable version"]
+       "/Print Version", "/print version", "/Printable version",
+       "/The Whole Book"]
       lem
-  where fun (y : _) x
+  where fun (y : ys) x
           = case splitOn y x of
-                (z : _) -> z
+                (z : _) -> fun ys z
                 _ -> []
         fun [] x = x
  
